"""The wrapping code for providing natural ctypes-based OpenGL interface"""
import ctypes
from OpenGL import platform, error
glGetError = platform.OpenGL.glGetError
from OpenGL import converters
from OpenGL.converters import DefaultCConverter
from OpenGL.converters import returnCArgument,returnPyArgument
NULL = object()

def asList( o ):
	"""Convert to a list if not already one"""
	if not isinstance( o, list ):
		return list(o)
	return o

class Wrapper( object ):
	"""Wrapper around a ctypes cFunction object providing SWIG-like hooks
	
	Attributes:
	
		wrappedOperation -- base operation, normally a ctypes function 
			with data-types and error-checking specified
		pyConverters -- converters for incoming Python arguments,
			provide 1:1 mapping to incoming Python arguments, can
			suppress an argument from the argument-set as well
				see setPyConverter
		pyConverterNames -- caching/storage of the argument names 
			for the Python converters 
		cConverters -- converters for incoming C-level arguments
			produce Python-level objects in 1:1 mapping to ctypes
			arguments from pyConverters results
				see setCConverter
		cResolvers -- converters turning Python-level objects into 
			ctypes-compatible data-types 
				see setCResolver
	
	Generic Attributes:
	
		{ARG1}_LOOKUP_{ARG2} -- lookup dictionaries to provide sizes for 
			ARG1 output value from the value of ARG2, provided for 
			documentation/reference
		{ARG1}_FROM_{ARG2} -- lookup functions to provide sizes for ARG1
			output value from the value of ARG2, provided for 
			documentation/reference
	"""
	localProperties = (
		'wrappedOperation', 
		'__file__',
		'pyConverters',
		'pyConverterNames',
		'cConverters',
		'cResolvers',
		'storeValues',
		'returnValues',
	)
	def __init__( self, wrappedOperation ):
		"""Initialise the wrapper, storing wrappedOperation"""
		if isinstance( wrappedOperation, Wrapper ):
			wrappedOperation = wrappedOperation.wrappedOperation
		self.wrappedOperation = wrappedOperation
	def __getattr__( self, key ):
		"""Delegate attribute lookup to our wrappedOperation"""
		if key != 'wrappedOperation':
			return getattr( self.wrappedOperation, key )
		raise AttributeError( key )
	def __nonzero__( self ):
		"""Is this function/wrapper available?"""
		return bool( self.wrappedOperation )
	def __setattr__( self, key, value ):
		"""Forward attribute setting to our wrappedOperation"""
		if key in self.localProperties:
			super( Wrapper, self ).__setattr__( key, value )
		else:
			return setattr( self.wrappedOperation, key, value )
	def pyArgIndex( self, argName ):
		"""Return the Python-argument index for the given argument name"""
		argNames = getattr( self, 'pyConverterNames', None )
		if argNames is None:
			argNames = self.wrappedOperation.argNames
		try:
			return asList( argNames ).index( argName )
		except (ValueError,IndexError), err:
			raise KeyError( """No argument %r in argument list %r"""%(
				argName, argNames
			))
	def cArgIndex( self, argName ):
		"""Return the C-argument index for the given argument name"""
		argNames = self.wrappedOperation.argNames
		try:
			return asList( argNames ).index( argName )
		except (ValueError,IndexError), err:
			raise KeyError( """No argument %r in argument list %r"""%(
				argName, argNames
			))
	def setOutput( 
		self, outArg, size=(1,), pnameArg=None, 
		arrayType=None, oldStyleReturn=True,
	):
		"""Set the given argName to be an output array
		
		size -- either a tuple compatible with arrayType.zeros or 
			a function taking pname to produce such a value.
		arrayType -- array data-type used to generate the output 
			array using the zeros class method...
		pname -- optional argument passed into size function, that 
			is, the name of the argument whose *value* will be passed 
			to the size function, often the name of an input argument 
			to be "sized" to match the output argument.
		"""
		if arrayType is None:
			# figure out from self.wrappedOperation's argtypes
			index = self.cArgIndex( outArg )
			arrayType = self.wrappedOperation.argtypes[ index ]
		if pnameArg is None:
			assert not callable( size )
			conv = converters.Output(
				name=outArg,
				size=size,
				arrayType=arrayType,
			)
		else:
			if isinstance( size, dict ):
				setattr( self, '%s_LOOKUP_%s'%(outArg,pnameArg), size )
				size = size.__getitem__
			else:
				setattr( self, '%s_FROM_%s'%(outArg,pnameArg), size )
			assert callable( size )
			conv = converters.SizedOutput(
				name=outArg,
				specifier=pnameArg,
				lookup=size,
				arrayType=arrayType,
			)
		if oldStyleReturn:
			returnObject = conv.oldStyleReturn
		else:
			returnObject = converters.returnCArgument( outArg )
		return self.setPyConverter(
			outArg
		).setCConverter(
			outArg, conv,
		).setReturnValues(
			returnObject
		)
	def setPyConverter( self, argName, function = NULL ):
		"""Set Python-argument converter for given argument
		
		argName -- the argument name which will be coerced to a usable internal
			format using the function provided.
		function -- None (indicating a simple copy), NULL (default) to eliminate
			the argument from the Python argument-list, or a callable object with 
			the signature:
			
				converter(arg, wrappedOperation, args)
			
			where arg is the particular argument on which the convert is working,
			wrappedOperation is the underlying wrapper, and args is the set of 
			original Python arguments to the function.
		
		Note that you need exactly the same number of pyConverters as Python 
		arguments.
		"""
		if not hasattr( self, 'pyConverters' ):
			self.pyConverters = [None]*len( self.wrappedOperation.argNames )
			self.pyConverterNames = list(self.wrappedOperation.argNames)
		try:
			i = asList( self.pyConverterNames ).index( argName )
		except ValueError:
			raise AttributeError( """No argument named %r left in pyConverters for %r: %s"""%(
				argName, self.wrappedOperation.__name__, self.pyConverterNames,
			))
		if function is NULL:
			del self.pyConverters[i]
			del self.pyConverterNames[i]
		else:
			self.pyConverters[i] = function 
		return self
	def setCConverter( self, argName, function ):
		"""Set C-argument converter for a given argument
		
		argName -- the argument name whose C-compatible representation will 
			be calculated with the passed function.
		function -- None (indicating a simple copy), a non-callable object to
			be copied into the result-list itself, or a callable object with 
			the signature:
			
				converter( pyArgs, index, wrappedOperation )
			
			where pyArgs is the set of passed Python arguments, with the 
			pyConverters already applied, index is the index of the C argument 
			and wrappedOperation is the underlying function.
		
		C-argument converters are your chance to expand/contract a Python 
		argument list (pyArgs) to match the number of arguments expected by
		the ctypes baseOperation.  You can't have a "null" C-argument converter,
		as *something* has to be passed to the C-level function in the 
		parameter.
		"""
		if not hasattr( self, 'cConverters' ):
			self.cConverters = [None]*len( self.wrappedOperation.argNames )
		try:
			if not isinstance(self.wrappedOperation.argNames, list):
				self.wrappedOperation.argNames = list( self.wrappedOperation.argNames )
			i = asList( self.wrappedOperation.argNames ).index( argName )
		except ValueError:
			raise AttributeError( """No argument named %r left in cConverters: %s"""%(
				argName, self.wrappedOperation.argNames,
			))
		self.cConverters[i] = function 
		return self
	def setCResolver( self, argName, function=NULL ):
		"""Set C-argument converter for a given argument"""
		if not hasattr( self, 'cResolvers' ):
			self.cResolvers = [None]*len( self.wrappedOperation.argNames )
		try:
			if not isinstance(self.wrappedOperation.argNames, list):
				self.wrappedOperation.argNames = list( self.wrappedOperation.argNames )
			i = asList( self.wrappedOperation.argNames).index( argName )
		except ValueError:
			raise AttributeError( """No argument named %r left in cConverters: %s"""%(
				argName, self.wrappedOperation.argNames,
			))
		if function is NULL:
			del self.cResolvers[i]
		else:
			self.cResolvers[i] = function 
		return self
	def setStoreValues( self, function=NULL ):
		"""Set the storage-of-arguments function for the whole wrapper"""
		if function is NULL:
			try:
				del self.storeValues
			except Exception, err:
				pass
		else:
			self.storeValues = function
		return self
	def setReturnValues( self, function=NULL ):
		"""Set the return-of-results function for the whole wrapper"""
		if function is NULL:
			try:
				del self.returnValues
			except Exception, err:
				pass
		else:
			self.returnValues = function
		return self
	def finalise( self ):
		"""Finalise our various elements into simple index-based operations"""
		for attribute in ('pyConverters','cConverters','cResolvers' ):
			value = getattr( self, attribute, None )
			if value is not None:
				for i,item in enumerate(value):
					if hasattr( item, 'finalise' ):
						try:
							item.finalise( self )
						except Exception, err:
							raise error.Error(
								"""Error finalising item %s in %s for %s (%r): %s"""%(
									i,attribute,self,item,err,
								)
							)
		if hasattr( self, 'cConverters' ):
			for i,converter in enumerate( self.cConverters ):
				if isinstance( converter, (type(None),DefaultCConverter )):
					self.cConverters[i] = DefaultCConverter( self.pyArgIndex( self.argNames[i]) )
		for attribute in ('storeValues','returnValues',):
			item = getattr( self, attribute, None )
			if hasattr( item, 'finalise' ):
				item.finalise( self )
		callFunction = self.finaliseCall()
		if not callFunction:
			raise RuntimeError( """Missing finalised call type for %s"""%( self, ))
		else:
			self.__class__.__call__ = callFunction
		return self
	def finaliseCall( self ):
		"""Produce specialised versions of call for finalised wrapper object
		
		This returns a version of __call__ that only does that work which is
		required by the particular wrapper object
		
		This is essentially a huge set of expanded nested functions, very 
		inelegant...
		"""
		pyConverters = getattr( self, 'pyConverters', None )
		cConverters = getattr( self, 'cConverters', None )
		cResolvers = getattr( self, 'cResolvers', None )
		wrappedOperation = self.wrappedOperation
		storeValues = getattr( self, 'storeValues', None )
		returnValues = getattr( self, 'returnValues', None )
		if pyConverters:
			if cConverters:
				if cResolvers:
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										try:
											cArguments.append( converter( value ) )
										except Exception, err:
											err.args += (converter,)
											raise
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return result 
							return wrapperCall
				else:
					# null cResolvers
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return result 
							return wrapperCall
			else:
				# null cConverters
				if cResolvers:
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = pyArgs
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = pyArgs
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = pyArgs
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArgs = pyArgs
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return result 
							return wrapperCall
				else:
					# null cResolvers
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArguments = pyArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArguments = pyArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArguments = pyArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								if len(pyConverters) != len(args):
									raise ValueError(
										"""%s requires %r arguments (%s), received %s: %r"""%(
											wrappedOperation.__name__,
											len(pyConverters),
											", ".join( self.pyConverterNames ),
											len(args),
											args
										)
									)
								pyArgs = []
								for (converter,arg) in zip(pyConverters,args):
									if converter is None:
										pyArgs.append( arg )
									else:
										pyArgs.append( converter(arg, self, args) )
								cArguments = pyArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return result 
							return wrapperCall
		else:
			# null pyConverters
			if cConverters:
				if cResolvers:
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return result 
							return wrapperCall
				else:
					# null cResolvers
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return returnValues(
									result,
									self,
									pyArgs,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								pyArgs = args 
								cArgs = []
								for (index,converter) in enumerate( cConverters ):
									# move enumerate out...
									if not callable(converter):
										cArgs.append( converter )
									else:
										try:
											cArgs.append(
												converter( pyArgs, index, self )
											)
										except Exception, err:
											if hasattr( err, 'args' ):
												err.args += (
													"""Failure in cConverter %r"""%(converter),
													pyArgs, index, self,
												)
											raise
								cArguments = cArgs
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = pyArgs
									raise err
								return result 
							return wrapperCall
			else:
				# null cConverters
				if cResolvers:
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								cArgs = args
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									args,
									cArgs,
								)
								return returnValues(
									result,
									self,
									args,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								cArgs = args
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									args,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								cArgs = args
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								return returnValues(
									result,
									self,
									args,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								cArgs = args
								cArguments = []
								for (converter, value) in zip( cResolvers, cArgs ):
									if converter is None:
										cArguments.append( value )
									else:
										cArguments.append( converter( value ) )
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								return result 
							return wrapperCall
				else:
					# null cResolvers
					if storeValues:
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all possible operations"""
								cArguments = args
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									args,
									cArgs,
								)
								return returnValues(
									result,
									self,
									args,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues"""
								cArguments = args
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								# handle storage of persistent argument values...
								storeValues(
									result,
									self,
									args,
									cArgs,
								)
								return result 
							return wrapperCall
					else: # null storeValues
						if returnValues:
							def wrapperCall( self, *args ):
								"""Wrapper with all save storeValues"""
								cArguments = args
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								return returnValues(
									result,
									self,
									args,
									cArgs,
								)
							return wrapperCall
						else:
							def wrapperCall( self, *args ):
								"""Wrapper with all save returnValues and storeValues"""
								cArguments = args
								try:
									result = self.wrappedOperation( *cArguments )
								except ctypes.ArgumentError, err:
									err.args = err.args + (cArguments,)
									raise err
								except error.GLError, err:
									err.cArgs = cArgs 
									err.pyArgs = args
									raise err
								return result 
							return wrapperCall
	def __call__( self, *args, **named ):
		"""Finalise the wrapper before calling it"""
		return self.finalise()( *args, **named )

	def _unspecialised__call__( self, *args ):
		"""Expand arguments, call the function, store values and check errors"""
		pyConverters = getattr( self, 'pyConverters', None )
		if pyConverters:
			if len(pyConverters) != len(args):
				raise ValueError(
					"""%s requires %r arguments (%s), received %s: %r"""%(
						self.wrappedOperation.__name__,
						len(pyConverters),
						", ".join( self.pyConverterNames ),
						len(args),
						args
					)
				)
			pyArgs = []
			for (converter,arg) in zip(pyConverters,args):
				if converter is None:
					pyArgs.append( arg )
				else:
					pyArgs.append( converter(arg, self, args) )
		else:
			pyArgs = args 
		cConverters = getattr( self, 'cConverters', None )
		if cConverters:
			cArgs = []
			for (index,converter) in enumerate( cConverters ):
				if not callable(converter):
					cArgs.append( converter )
				else:
					try:
						cArgs.append(
							converter( pyArgs, index, self )
						)
					except Exception, err:
						if hasattr( err, 'args' ):
							err.args += (
								"""Failure in cConverter %r"""%(converter),
								pyArgs, index, self,
							)
						raise
		else:
			cArgs = pyArgs
		cResolvers = getattr( self, 'cResolvers', None )
		if cResolvers:
			cArguments = []
			for (converter, value) in zip( cResolvers, cArgs ):
				if converter is None:
					cArguments.append( value )
				else:
					cArguments.append( converter( value ) )
		else:
			cArguments = cArgs
		try:
			result = self.wrappedOperation( *cArguments )
		except ctypes.ArgumentError, err:
			err.args = err.args + (cArguments,)
			raise err
		except error.GLError, err:
			err.cArgs = cArgs 
			err.pyArgs = pyArgs
			raise err
		storeValues = getattr( self, 'storeValues', None )
		if storeValues is not None:
			# handle storage of persistent argument values...
			storeValues(
				result,
				self,
				pyArgs,
				cArgs,
			)
		returnValues = getattr( self, 'returnValues', None )
		if returnValues is not None:
			return returnValues(
				result,
				self,
				pyArgs,
				cArgs,
			)
		else:
			return result

def wrapper( wrappedOperation ):
	"""Create a Wrapper sub-class instance for the given wrappedOperation
	
	The purpose of this function is to create a subclass of Wrapper which
	has the __doc__ and __name__ of the wrappedOperation so that the instance of
	the wrapper will show up as <functionname instance @ address> by default,
	and will have the docstring available naturally in pydoc and the like.
	"""
	if isinstance( wrappedOperation, Wrapper ):
		return wrappedOperation
	dict = {
		'__doc__': wrappedOperation.__doc__,
		'__slots__': ('wrappedOperation', ),
	}
	cls = type( wrappedOperation.__name__, (Wrapper,), dict )
	instance = cls(wrappedOperation)
	return instance



'''OpenGL extension EXT.histogram

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.histogram to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.histogram import *
### END AUTOGENERATED SECTION

glGetHistogramParameterfvEXT = wrapper.wrapper(glGetHistogramParameterfvEXT).setOutput(
	"params",(1,), 
)
glGetHistogramParameterivEXT = wrapper.wrapper(glGetHistogramParameterivEXT).setOutput(
	"params",(1,), 
)



'''OpenGL extension ARB.occlusion_query

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.occlusion_query to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.occlusion_query import *
### END AUTOGENERATED SECTION
from OpenGL.GL import glget
from OpenGL import converters
for func in ('glGenQueriesARB','glDeleteQueriesARB'):
	globals()[func] = wrapper.wrapper( 
		globals()[func],
	).setPyConverter('n').setCConverter(
		'n', arrays.AsArrayTypedSize( 'ids', arrays.GLuintArray ),
	).setCConverter(
		'ids', arrays.asArrayType(arrays.GLuintArray),
	).setReturnValues(
		wrapper.returnPyArgument( 'ids' )
	)

for func in (
	'glGetQueryivARB','glGetQueryObjectivARB','glGetQueryObjectuivARB',
):
	globals()[func] = wrapper.wrapper(globals()[func]).setOutput(
		"params", (1,)
	)
del func, glget



"""OpenGL version 1.3 imaging-handling routines"""
from OpenGL import wrapper, constants, arrays
from OpenGL.raw.GL.VERSION import GL_1_3 as simple
from OpenGL.GL import images, glget

for dimensions in (1,2,3):
	for function in ('glCompressedTexImage%sD','glCompressedTexSubImage%sD'):
		name = function%(dimensions,)
		globals()[ name ] = images.compressedImageFunction(
			getattr( simple, name )
		)
		del name, function
	del dimensions

if simple.glGetCompressedTexImage:
	def glGetCompressedTexImage( target, level, img=None ):
		"""Retrieve a compressed texture image"""
		if img is None:
			length = glget.glGetTexLevelParameteriv(
				target, 0, 
				simple.GL_TEXTURE_COMPRESSED_IMAGE_SIZE_ARB, 
			)
			img = arrays.ArrayDataType.zeros( (length,), constants.GL_UNSIGNED_BYTE )
		return simple.glGetCompressedTexImage(target, 0, img);



'''OpenGL extension EXT.vertex_shader

Overview (from the spec)
	
	EXT_vertex_shader adds a flexible way to change the per-vertex
	processing in the GL pipeline. It provides a method to replace
	the fixed vertex/normal transform and lighting with a user
	specified means of generating processed vertices, texture
	coordinates, color, and secondary color, along with a primitive's
	associated state.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/vertex_shader.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_vertex_shader'
GL_VERTEX_SHADER_EXT = constant.Constant( 'GL_VERTEX_SHADER_EXT', 0x8780 )
glget.addGLGetConstant( GL_VERTEX_SHADER_EXT, (1,) )
GL_VERTEX_SHADER_BINDING_EXT = constant.Constant( 'GL_VERTEX_SHADER_BINDING_EXT', 0x8781 )
GL_OP_INDEX_EXT = constant.Constant( 'GL_OP_INDEX_EXT', 0x8782 )
GL_OP_NEGATE_EXT = constant.Constant( 'GL_OP_NEGATE_EXT', 0x8783 )
GL_OP_DOT3_EXT = constant.Constant( 'GL_OP_DOT3_EXT', 0x8784 )
GL_OP_DOT4_EXT = constant.Constant( 'GL_OP_DOT4_EXT', 0x8785 )
GL_OP_MUL_EXT = constant.Constant( 'GL_OP_MUL_EXT', 0x8786 )
GL_OP_ADD_EXT = constant.Constant( 'GL_OP_ADD_EXT', 0x8787 )
GL_OP_MADD_EXT = constant.Constant( 'GL_OP_MADD_EXT', 0x8788 )
GL_OP_FRAC_EXT = constant.Constant( 'GL_OP_FRAC_EXT', 0x8789 )
GL_OP_MAX_EXT = constant.Constant( 'GL_OP_MAX_EXT', 0x878A )
GL_OP_MIN_EXT = constant.Constant( 'GL_OP_MIN_EXT', 0x878B )
GL_OP_SET_GE_EXT = constant.Constant( 'GL_OP_SET_GE_EXT', 0x878C )
GL_OP_SET_LT_EXT = constant.Constant( 'GL_OP_SET_LT_EXT', 0x878D )
GL_OP_CLAMP_EXT = constant.Constant( 'GL_OP_CLAMP_EXT', 0x878E )
GL_OP_FLOOR_EXT = constant.Constant( 'GL_OP_FLOOR_EXT', 0x878F )
GL_OP_ROUND_EXT = constant.Constant( 'GL_OP_ROUND_EXT', 0x8790 )
GL_OP_EXP_BASE_2_EXT = constant.Constant( 'GL_OP_EXP_BASE_2_EXT', 0x8791 )
GL_OP_LOG_BASE_2_EXT = constant.Constant( 'GL_OP_LOG_BASE_2_EXT', 0x8792 )
GL_OP_POWER_EXT = constant.Constant( 'GL_OP_POWER_EXT', 0x8793 )
GL_OP_RECIP_EXT = constant.Constant( 'GL_OP_RECIP_EXT', 0x8794 )
GL_OP_RECIP_SQRT_EXT = constant.Constant( 'GL_OP_RECIP_SQRT_EXT', 0x8795 )
GL_OP_SUB_EXT = constant.Constant( 'GL_OP_SUB_EXT', 0x8796 )
GL_OP_CROSS_PRODUCT_EXT = constant.Constant( 'GL_OP_CROSS_PRODUCT_EXT', 0x8797 )
GL_OP_MULTIPLY_MATRIX_EXT = constant.Constant( 'GL_OP_MULTIPLY_MATRIX_EXT', 0x8798 )
GL_OP_MOV_EXT = constant.Constant( 'GL_OP_MOV_EXT', 0x8799 )
GL_OUTPUT_VERTEX_EXT = constant.Constant( 'GL_OUTPUT_VERTEX_EXT', 0x879A )
GL_OUTPUT_COLOR0_EXT = constant.Constant( 'GL_OUTPUT_COLOR0_EXT', 0x879B )
GL_OUTPUT_COLOR1_EXT = constant.Constant( 'GL_OUTPUT_COLOR1_EXT', 0x879C )
GL_OUTPUT_TEXTURE_COORD0_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD0_EXT', 0x879D )
GL_OUTPUT_TEXTURE_COORD1_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD1_EXT', 0x879E )
GL_OUTPUT_TEXTURE_COORD2_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD2_EXT', 0x879F )
GL_OUTPUT_TEXTURE_COORD3_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD3_EXT', 0x87A0 )
GL_OUTPUT_TEXTURE_COORD4_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD4_EXT', 0x87A1 )
GL_OUTPUT_TEXTURE_COORD5_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD5_EXT', 0x87A2 )
GL_OUTPUT_TEXTURE_COORD6_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD6_EXT', 0x87A3 )
GL_OUTPUT_TEXTURE_COORD7_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD7_EXT', 0x87A4 )
GL_OUTPUT_TEXTURE_COORD8_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD8_EXT', 0x87A5 )
GL_OUTPUT_TEXTURE_COORD9_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD9_EXT', 0x87A6 )
GL_OUTPUT_TEXTURE_COORD10_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD10_EXT', 0x87A7 )
GL_OUTPUT_TEXTURE_COORD11_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD11_EXT', 0x87A8 )
GL_OUTPUT_TEXTURE_COORD12_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD12_EXT', 0x87A9 )
GL_OUTPUT_TEXTURE_COORD13_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD13_EXT', 0x87AA )
GL_OUTPUT_TEXTURE_COORD14_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD14_EXT', 0x87AB )
GL_OUTPUT_TEXTURE_COORD15_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD15_EXT', 0x87AC )
GL_OUTPUT_TEXTURE_COORD16_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD16_EXT', 0x87AD )
GL_OUTPUT_TEXTURE_COORD17_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD17_EXT', 0x87AE )
GL_OUTPUT_TEXTURE_COORD18_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD18_EXT', 0x87AF )
GL_OUTPUT_TEXTURE_COORD19_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD19_EXT', 0x87B0 )
GL_OUTPUT_TEXTURE_COORD20_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD20_EXT', 0x87B1 )
GL_OUTPUT_TEXTURE_COORD21_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD21_EXT', 0x87B2 )
GL_OUTPUT_TEXTURE_COORD22_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD22_EXT', 0x87B3 )
GL_OUTPUT_TEXTURE_COORD23_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD23_EXT', 0x87B4 )
GL_OUTPUT_TEXTURE_COORD24_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD24_EXT', 0x87B5 )
GL_OUTPUT_TEXTURE_COORD25_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD25_EXT', 0x87B6 )
GL_OUTPUT_TEXTURE_COORD26_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD26_EXT', 0x87B7 )
GL_OUTPUT_TEXTURE_COORD27_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD27_EXT', 0x87B8 )
GL_OUTPUT_TEXTURE_COORD28_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD28_EXT', 0x87B9 )
GL_OUTPUT_TEXTURE_COORD29_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD29_EXT', 0x87BA )
GL_OUTPUT_TEXTURE_COORD30_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD30_EXT', 0x87BB )
GL_OUTPUT_TEXTURE_COORD31_EXT = constant.Constant( 'GL_OUTPUT_TEXTURE_COORD31_EXT', 0x87BC )
GL_OUTPUT_FOG_EXT = constant.Constant( 'GL_OUTPUT_FOG_EXT', 0x87BD )
GL_SCALAR_EXT = constant.Constant( 'GL_SCALAR_EXT', 0x87BE )
GL_VECTOR_EXT = constant.Constant( 'GL_VECTOR_EXT', 0x87BF )
GL_MATRIX_EXT = constant.Constant( 'GL_MATRIX_EXT', 0x87C0 )
GL_VARIANT_EXT = constant.Constant( 'GL_VARIANT_EXT', 0x87C1 )
GL_INVARIANT_EXT = constant.Constant( 'GL_INVARIANT_EXT', 0x87C2 )
GL_LOCAL_CONSTANT_EXT = constant.Constant( 'GL_LOCAL_CONSTANT_EXT', 0x87C3 )
GL_LOCAL_EXT = constant.Constant( 'GL_LOCAL_EXT', 0x87C4 )
GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT = constant.Constant( 'GL_MAX_VERTEX_SHADER_INSTRUCTIONS_EXT', 0x87C5 )
GL_MAX_VERTEX_SHADER_VARIANTS_EXT = constant.Constant( 'GL_MAX_VERTEX_SHADER_VARIANTS_EXT', 0x87C6 )
GL_MAX_VERTEX_SHADER_INVARIANTS_EXT = constant.Constant( 'GL_MAX_VERTEX_SHADER_INVARIANTS_EXT', 0x87C7 )
GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = constant.Constant( 'GL_MAX_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 0x87C8 )
GL_MAX_VERTEX_SHADER_LOCALS_EXT = constant.Constant( 'GL_MAX_VERTEX_SHADER_LOCALS_EXT', 0x87C9 )
GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT = constant.Constant( 'GL_MAX_OPTIMIZED_VERTEX_SHADER_INSTRUCTIONS_EXT', 0x87CA )
GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT = constant.Constant( 'GL_MAX_OPTIMIZED_VERTEX_SHADER_VARIANTS_EXT', 0x87CB )
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = constant.Constant( 'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 0x87CC )
GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT = constant.Constant( 'GL_MAX_OPTIMIZED_VERTEX_SHADER_INVARIANTS_EXT', 0x87CD )
GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT = constant.Constant( 'GL_MAX_OPTIMIZED_VERTEX_SHADER_LOCALS_EXT', 0x87CE )
GL_VERTEX_SHADER_INSTRUCTIONS_EXT = constant.Constant( 'GL_VERTEX_SHADER_INSTRUCTIONS_EXT', 0x87CF )
GL_VERTEX_SHADER_VARIANTS_EXT = constant.Constant( 'GL_VERTEX_SHADER_VARIANTS_EXT', 0x87D0 )
GL_VERTEX_SHADER_INVARIANTS_EXT = constant.Constant( 'GL_VERTEX_SHADER_INVARIANTS_EXT', 0x87D1 )
GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT = constant.Constant( 'GL_VERTEX_SHADER_LOCAL_CONSTANTS_EXT', 0x87D2 )
GL_VERTEX_SHADER_LOCALS_EXT = constant.Constant( 'GL_VERTEX_SHADER_LOCALS_EXT', 0x87D3 )
GL_VERTEX_SHADER_OPTIMIZED_EXT = constant.Constant( 'GL_VERTEX_SHADER_OPTIMIZED_EXT', 0x87D4 )
glget.addGLGetConstant( GL_VERTEX_SHADER_OPTIMIZED_EXT, (1,) )
GL_X_EXT = constant.Constant( 'GL_X_EXT', 0x87D5 )
GL_Y_EXT = constant.Constant( 'GL_Y_EXT', 0x87D6 )
GL_Z_EXT = constant.Constant( 'GL_Z_EXT', 0x87D7 )
GL_W_EXT = constant.Constant( 'GL_W_EXT', 0x87D8 )
GL_NEGATIVE_X_EXT = constant.Constant( 'GL_NEGATIVE_X_EXT', 0x87D9 )
GL_NEGATIVE_Y_EXT = constant.Constant( 'GL_NEGATIVE_Y_EXT', 0x87DA )
GL_NEGATIVE_Z_EXT = constant.Constant( 'GL_NEGATIVE_Z_EXT', 0x87DB )
GL_NEGATIVE_W_EXT = constant.Constant( 'GL_NEGATIVE_W_EXT', 0x87DC )
GL_ZERO_EXT = constant.Constant( 'GL_ZERO_EXT', 0x87DD )
GL_ONE_EXT = constant.Constant( 'GL_ONE_EXT', 0x87DE )
GL_NEGATIVE_ONE_EXT = constant.Constant( 'GL_NEGATIVE_ONE_EXT', 0x87DF )
GL_NORMALIZED_RANGE_EXT = constant.Constant( 'GL_NORMALIZED_RANGE_EXT', 0x87E0 )
GL_FULL_RANGE_EXT = constant.Constant( 'GL_FULL_RANGE_EXT', 0x87E1 )
GL_CURRENT_VERTEX_EXT = constant.Constant( 'GL_CURRENT_VERTEX_EXT', 0x87E2 )
GL_MVP_MATRIX_EXT = constant.Constant( 'GL_MVP_MATRIX_EXT', 0x87E3 )
GL_VARIANT_VALUE_EXT = constant.Constant( 'GL_VARIANT_VALUE_EXT', 0x87E4 )
GL_VARIANT_DATATYPE_EXT = constant.Constant( 'GL_VARIANT_DATATYPE_EXT', 0x87E5 )
GL_VARIANT_ARRAY_STRIDE_EXT = constant.Constant( 'GL_VARIANT_ARRAY_STRIDE_EXT', 0x87E6 )
GL_VARIANT_ARRAY_TYPE_EXT = constant.Constant( 'GL_VARIANT_ARRAY_TYPE_EXT', 0x87E7 )
GL_VARIANT_ARRAY_EXT = constant.Constant( 'GL_VARIANT_ARRAY_EXT', 0x87E8 )
GL_VARIANT_ARRAY_POINTER_EXT = constant.Constant( 'GL_VARIANT_ARRAY_POINTER_EXT', 0x87E9 )
GL_INVARIANT_VALUE_EXT = constant.Constant( 'GL_INVARIANT_VALUE_EXT', 0x87EA )
GL_INVARIANT_DATATYPE_EXT = constant.Constant( 'GL_INVARIANT_DATATYPE_EXT', 0x87EB )
GL_LOCAL_CONSTANT_VALUE_EXT = constant.Constant( 'GL_LOCAL_CONSTANT_VALUE_EXT', 0x87EC )
GL_LOCAL_CONSTANT_DATATYPE_EXT = constant.Constant( 'GL_LOCAL_CONSTANT_DATATYPE_EXT', 0x87ED )
glBeginVertexShaderEXT = platform.createExtensionFunction( 
	'glBeginVertexShaderEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glBeginVertexShaderEXT(  ) -> None',
	argNames = (),
)

glEndVertexShaderEXT = platform.createExtensionFunction( 
	'glEndVertexShaderEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glEndVertexShaderEXT(  ) -> None',
	argNames = (),
)

glBindVertexShaderEXT = platform.createExtensionFunction( 
	'glBindVertexShaderEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glBindVertexShaderEXT( GLuint(id) ) -> None',
	argNames = ('id',),
)

glGenVertexShadersEXT = platform.createExtensionFunction( 
	'glGenVertexShadersEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLuint,),
	doc = 'glGenVertexShadersEXT( GLuint(range) ) -> constants.GLuint',
	argNames = ('range',),
)

glDeleteVertexShaderEXT = platform.createExtensionFunction( 
	'glDeleteVertexShaderEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glDeleteVertexShaderEXT( GLuint(id) ) -> None',
	argNames = ('id',),
)

glShaderOp1EXT = platform.createExtensionFunction( 
	'glShaderOp1EXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint,),
	doc = 'glShaderOp1EXT( GLenum(op), GLuint(res), GLuint(arg1) ) -> None',
	argNames = ('op', 'res', 'arg1',),
)

glShaderOp2EXT = platform.createExtensionFunction( 
	'glShaderOp2EXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glShaderOp2EXT( GLenum(op), GLuint(res), GLuint(arg1), GLuint(arg2) ) -> None',
	argNames = ('op', 'res', 'arg1', 'arg2',),
)

glShaderOp3EXT = platform.createExtensionFunction( 
	'glShaderOp3EXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glShaderOp3EXT( GLenum(op), GLuint(res), GLuint(arg1), GLuint(arg2), GLuint(arg3) ) -> None',
	argNames = ('op', 'res', 'arg1', 'arg2', 'arg3',),
)

glSwizzleEXT = platform.createExtensionFunction( 
	'glSwizzleEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glSwizzleEXT( GLuint(res), GLuint(in), GLenum(outX), GLenum(outY), GLenum(outZ), GLenum(outW) ) -> None',
	argNames = ('res', 'in', 'outX', 'outY', 'outZ', 'outW',),
)

glWriteMaskEXT = platform.createExtensionFunction( 
	'glWriteMaskEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLenum, constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glWriteMaskEXT( GLuint(res), GLuint(in), GLenum(outX), GLenum(outY), GLenum(outZ), GLenum(outW) ) -> None',
	argNames = ('res', 'in', 'outX', 'outY', 'outZ', 'outW',),
)

glInsertComponentEXT = platform.createExtensionFunction( 
	'glInsertComponentEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glInsertComponentEXT( GLuint(res), GLuint(src), GLuint(num) ) -> None',
	argNames = ('res', 'src', 'num',),
)

glExtractComponentEXT = platform.createExtensionFunction( 
	'glExtractComponentEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glExtractComponentEXT( GLuint(res), GLuint(src), GLuint(num) ) -> None',
	argNames = ('res', 'src', 'num',),
)

glGenSymbolsEXT = platform.createExtensionFunction( 
	'glGenSymbolsEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum, constants.GLuint,),
	doc = 'glGenSymbolsEXT( GLenum(datatype), GLenum(storagetype), GLenum(range), GLuint(components) ) -> constants.GLuint',
	argNames = ('datatype', 'storagetype', 'range', 'components',),
)

glSetInvariantEXT = platform.createExtensionFunction( 
	'glSetInvariantEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.c_void_p,),
	doc = 'glSetInvariantEXT( GLuint(id), GLenum(type), c_void_p(addr) ) -> None',
	argNames = ('id', 'type', 'addr',),
)

glSetLocalConstantEXT = platform.createExtensionFunction( 
	'glSetLocalConstantEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.c_void_p,),
	doc = 'glSetLocalConstantEXT( GLuint(id), GLenum(type), c_void_p(addr) ) -> None',
	argNames = ('id', 'type', 'addr',),
)

glVariantbvEXT = platform.createExtensionFunction( 
	'glVariantbvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLbyteArray,),
	doc = 'glVariantbvEXT( GLuint(id), GLbyteArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantsvEXT = platform.createExtensionFunction( 
	'glVariantsvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLshortArray,),
	doc = 'glVariantsvEXT( GLuint(id), GLshortArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantivEXT = platform.createExtensionFunction( 
	'glVariantivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLintArray,),
	doc = 'glVariantivEXT( GLuint(id), GLintArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantfvEXT = platform.createExtensionFunction( 
	'glVariantfvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLfloatArray,),
	doc = 'glVariantfvEXT( GLuint(id), GLfloatArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantdvEXT = platform.createExtensionFunction( 
	'glVariantdvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLdoubleArray,),
	doc = 'glVariantdvEXT( GLuint(id), GLdoubleArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantubvEXT = platform.createExtensionFunction( 
	'glVariantubvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLubyteArray,),
	doc = 'glVariantubvEXT( GLuint(id), GLubyteArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantusvEXT = platform.createExtensionFunction( 
	'glVariantusvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVariantusvEXT( GLuint(id), GLushortArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantuivEXT = platform.createExtensionFunction( 
	'glVariantuivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLuintArray,),
	doc = 'glVariantuivEXT( GLuint(id), GLuintArray(addr) ) -> None',
	argNames = ('id', 'addr',),
)

glVariantPointerEXT = platform.createExtensionFunction( 
	'glVariantPointerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, constants.GLuint, ctypes.c_void_p,),
	doc = 'glVariantPointerEXT( GLuint(id), GLenum(type), GLuint(stride), c_void_p(addr) ) -> None',
	argNames = ('id', 'type', 'stride', 'addr',),
)

glEnableVariantClientStateEXT = platform.createExtensionFunction( 
	'glEnableVariantClientStateEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glEnableVariantClientStateEXT( GLuint(id) ) -> None',
	argNames = ('id',),
)

glDisableVariantClientStateEXT = platform.createExtensionFunction( 
	'glDisableVariantClientStateEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glDisableVariantClientStateEXT( GLuint(id) ) -> None',
	argNames = ('id',),
)

glBindLightParameterEXT = platform.createExtensionFunction( 
	'glBindLightParameterEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glBindLightParameterEXT( GLenum(light), GLenum(value) ) -> constants.GLuint',
	argNames = ('light', 'value',),
)

glBindMaterialParameterEXT = platform.createExtensionFunction( 
	'glBindMaterialParameterEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glBindMaterialParameterEXT( GLenum(face), GLenum(value) ) -> constants.GLuint',
	argNames = ('face', 'value',),
)

glBindTexGenParameterEXT = platform.createExtensionFunction( 
	'glBindTexGenParameterEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum, constants.GLenum, constants.GLenum,),
	doc = 'glBindTexGenParameterEXT( GLenum(unit), GLenum(coord), GLenum(value) ) -> constants.GLuint',
	argNames = ('unit', 'coord', 'value',),
)

glBindTextureUnitParameterEXT = platform.createExtensionFunction( 
	'glBindTextureUnitParameterEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum, constants.GLenum,),
	doc = 'glBindTextureUnitParameterEXT( GLenum(unit), GLenum(value) ) -> constants.GLuint',
	argNames = ('unit', 'value',),
)

glBindParameterEXT = platform.createExtensionFunction( 
	'glBindParameterEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLuint, 
	argTypes=(constants.GLenum,),
	doc = 'glBindParameterEXT( GLenum(value) ) -> constants.GLuint',
	argNames = ('value',),
)

glIsVariantEnabledEXT = platform.createExtensionFunction( 
	'glIsVariantEnabledEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint, constants.GLenum,),
	doc = 'glIsVariantEnabledEXT( GLuint(id), GLenum(cap) ) -> constants.GLboolean',
	argNames = ('id', 'cap',),
)

glGetVariantBooleanvEXT = platform.createExtensionFunction( 
	'glGetVariantBooleanvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(constants.GLboolean),),
	doc = 'glGetVariantBooleanvEXT( GLuint(id), GLenum(value), POINTER(constants.GLboolean)(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetVariantIntegervEXT = platform.createExtensionFunction( 
	'glGetVariantIntegervEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetVariantIntegervEXT( GLuint(id), GLenum(value), GLintArray(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetVariantFloatvEXT = platform.createExtensionFunction( 
	'glGetVariantFloatvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetVariantFloatvEXT( GLuint(id), GLenum(value), GLfloatArray(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetVariantPointervEXT = platform.createExtensionFunction( 
	'glGetVariantPointervEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glGetVariantPointervEXT( GLuint(id), GLenum(value), POINTER(ctypes.c_void_p)(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetInvariantBooleanvEXT = platform.createExtensionFunction( 
	'glGetInvariantBooleanvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(constants.GLboolean),),
	doc = 'glGetInvariantBooleanvEXT( GLuint(id), GLenum(value), POINTER(constants.GLboolean)(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetInvariantIntegervEXT = platform.createExtensionFunction( 
	'glGetInvariantIntegervEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetInvariantIntegervEXT( GLuint(id), GLenum(value), GLintArray(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetInvariantFloatvEXT = platform.createExtensionFunction( 
	'glGetInvariantFloatvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetInvariantFloatvEXT( GLuint(id), GLenum(value), GLfloatArray(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetLocalConstantBooleanvEXT = platform.createExtensionFunction( 
	'glGetLocalConstantBooleanvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, ctypes.POINTER(constants.GLboolean),),
	doc = 'glGetLocalConstantBooleanvEXT( GLuint(id), GLenum(value), POINTER(constants.GLboolean)(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetLocalConstantIntegervEXT = platform.createExtensionFunction( 
	'glGetLocalConstantIntegervEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetLocalConstantIntegervEXT( GLuint(id), GLenum(value), GLintArray(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)

glGetLocalConstantFloatvEXT = platform.createExtensionFunction( 
	'glGetLocalConstantFloatvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetLocalConstantFloatvEXT( GLuint(id), GLenum(value), GLfloatArray(data) ) -> None',
	argNames = ('id', 'value', 'data',),
)


def glInitVertexShaderEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shading_language_100

Overview (from the spec)
	
	    This extension string indicates that the OpenGL Shading Language is
	    supported. The Shading Language is defined by a separate specification
	    document which can be downloaded from
	
		http://www.opengl.org/documentation/oglsl.html

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/shading_language_100.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_shading_language_100'
GL_SHADING_LANGUAGE_VERSION_ARB = constant.Constant( 'GL_SHADING_LANGUAGE_VERSION_ARB', 0x8B8C )


def glInitShadingLanguage100ARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.pixel_format_float

Overview (from the spec)
	
	This extension adds pixel formats with floating-point RGBA color
	components.
	
	The size of each float components is specified using the same
	WGL_RED_BITS_ARB, WGL_GREEN_BITS_ARB, WGL_BLUE_BITS_ARB and
	WGL_ALPHA_BITS_ARB pixel format attributes that are used for
	defining the size of fixed-point components.  32 bit floating-
	point components are in the standard IEEE float format.  16 bit
	floating-point components have 1 sign bit, 5 exponent bits,
	and 10 mantissa bits. 
	
	In standard OpenGL RGBA color components are normally clamped to
	the range [0,1].  The color components of a float buffer are
	clamped to the limits of the range representable by their format.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/pixel_format_float.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_pixel_format_float'
GL_TYPE_RGBA_FLOAT_ATI = constant.Constant( 'GL_TYPE_RGBA_FLOAT_ATI', 0x8820 )
GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI = constant.Constant( 'GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI', 0x8835 )
glget.addGLGetConstant( GL_COLOR_CLEAR_UNCLAMPED_VALUE_ATI, (4,) )


def glInitPixelFormatFloatATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



import numpy
import PyQt4.Qt as qt

try:
    import OpenGL.GL  as GL
    import OpenGL.GLU as GLU
except ImportError:
    raise ImportError, "OpenGL must be installed to use these functionalities"


class MyGLWidget(qt.QGLWidget):
    def __init__(self, parent = None, crash = 0):
        qt.QGLWidget.__init__(self, parent)
        if crash:
            self._t = numpy.float64
        else:
            self._t = numpy.float32
        self.crash = crash
        self.generateData(512, 512)

    def generateData(self, xsize, ysize):
        self.xsize = xsize
        self.ysize = ysize
        self.vertices = numpy.zeros((xsize * ysize, 3), self._t)
        x = numpy.arange(xsize)
        y = numpy.arange(ysize)
        A=numpy.outer(x, numpy.ones(len(y), numpy.float32))
        B=numpy.outer(y, numpy.ones(len(x), numpy.float32))
        self.vertices[:,0]=A.flatten()
        self.vertices[:,1]=B.transpose().flatten()
        self.vertices[:,2]=range(xsize * ysize)
        self.vertexColors = numpy.zeros((xsize*ysize, 4), self._t)
        i = numpy.arange(len(self.vertices))
        self.vertexColors[:, 0] = (i & 255) / 255.
        self.vertexColors[:, 1] = ((i >> 8) & 255) / 255.
        self.vertexColors[:, 2] = ((i >> 16) & 255) / 255.
        self.vertexColors[:, 3] = 1.0

    def initializeGL(self):
        GL.glClearDepth(1.0)
        GL.glClearColor(0.5, 0.5, 0.5, 1.0)

        GL.glEnable(GL.GL_BLEND)
        GL.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA)

        GL.glShadeModel(GL.GL_FLAT)
        GL.glDisable(GL.GL_DITHER)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glDisable(GL.GL_CULL_FACE)

    def resizeGL(self, width, height):
        GL.glViewport(0, 0, width, height)


        GL.glMatrixMode(GL.GL_PROJECTION)
        GL.glLoadIdentity()
        GL.glOrtho(0., 512., 0., 512., -self.xsize*self.ysize, self.xsize*self.ysize)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glLoadIdentity()

        self.updateGL()


    def paintGL(self):
        GL.glClearColor(0.5, 0.5, 0.5, 1.0)
        GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        GL.glMatrixMode(GL.GL_MODELVIEW)
        GL.glLoadIdentity()
        self.drawObject()
        color = GL.glReadPixelsub(0, 0, 1, 1, GL.GL_RGBA)
        if color.dtype == 'int8':
            print "WARNING: int8 received"

    def drawObject(self):
        GL.glVertexPointerf(self.vertices)
        if self.crash != 2:
            GL.glColorPointerf(self.vertexColors)
        else:
            GL.glColorPointerd(self.vertexColors)
        GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
        GL.glEnableClientState(GL.GL_COLOR_ARRAY)
        GL.glDrawArrays(GL.GL_POINTS, 0, self.xsize*self.ysize)
        GL.glDisableClientState(GL.GL_COLOR_ARRAY)
        GL.glDisableClientState(GL.GL_VERTEX_ARRAY)

    def mousePressEvent(self, event):
        self.lastPos = qt.QPoint(event.pos())
        #get the color
        x = self.lastPos.x()
        y = self.lastPos.y()
        y = self.height()- y
        self.makeCurrent()
        color = GL.glReadPixelsub(x, y, 1, 1, GL.GL_RGBA)
        #workaround a PyOpenGL bug
        if color.dtype == 'int8':
            print "WARNING: int8 received"
            color = color.astype(numpy.uint8)
        else:
            print "received %s" % color.dtype
        print "Color = ", color

if __name__ == "__main__":
    import sys
    if len(sys.argv) < 2:
        print "Usage:"
        print "python OpenGlTest.py 0 to get it going"
        print "python OpenGlTest.py 1 to get it crash"
        print "python OpenGlTest.py 2 to get it thru again using glColorPointerd"
        sys.exit(0)
    app = qt.QApplication(sys.argv)
    value = int(sys.argv[1])
    window = MyGLWidget(crash=value)
    window.show()
    app.exec_()


                



'''OpenGL extension MESA.pack_invert

Overview (from the spec)
	
	This extension adds a new pixel storage parameter to indicate that
	images are to be packed in top-to-bottom order instead of OpenGL's
	conventional bottom-to-top order.  Only pixel packing can be
	inverted (i.e. for glReadPixels, glGetTexImage, glGetConvolutionFilter,
	etc).
	
	Almost all known image file formats store images in top-to-bottom
	order.  As it is, OpenGL reads images from the frame buffer in
	bottom-to-top order.  Thus, images usually have to be inverted before
	writing them to a file with image I/O libraries.  This extension
	allows images to be read such that inverting isn't needed.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/MESA/pack_invert.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_MESA_pack_invert'
GL_PACK_INVERT_MESA = constant.Constant( 'GL_PACK_INVERT_MESA', 0x8758 )
glget.addGLGetConstant( GL_PACK_INVERT_MESA, (1,) )


def glInitPackInvertMESA():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.resample

Overview (from the spec)
	
	This extension enhances the unpacking resampling capabilities
	of the SGIX_subsample extension.
	
	When pixel data is received from the client and an unpacking
	upsampling mode other than PIXEL_SUBSAMPLE_RATE_4444_SGIX is
	specified, the upsampling is performed via one of two methods:
	RESAMPLE_REPLICATE_SGIX, RESAMPLE_ZERO_FILL_SGIX.
	Replicate and zero fill are provided to
	give the application greatest performance and control over the
	filtering process.
	
	However, when pixel data is read back to the client and a
	packing downsampling mode other than PIXEL_SUBSAMPLE_RATE_4444_SGIX
	is specified, downsampling is
	performed via simple component decimation (point sampling). That is,
	only the RESAMPLE_DECIMATE_SGIX is valid.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/resample.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_resample'
GL_PACK_RESAMPLE_SGIX = constant.Constant( 'GL_PACK_RESAMPLE_SGIX', 0x842C )
glget.addGLGetConstant( GL_PACK_RESAMPLE_SGIX, (1,) )
GL_UNPACK_RESAMPLE_SGIX = constant.Constant( 'GL_UNPACK_RESAMPLE_SGIX', 0x842D )
glget.addGLGetConstant( GL_UNPACK_RESAMPLE_SGIX, (1,) )
GL_RESAMPLE_REPLICATE_SGIX = constant.Constant( 'GL_RESAMPLE_REPLICATE_SGIX', 0x842E )
GL_RESAMPLE_ZERO_FILL_SGIX = constant.Constant( 'GL_RESAMPLE_ZERO_FILL_SGIX', 0x842F )
GL_RESAMPLE_DECIMATE_SGIX = constant.Constant( 'GL_RESAMPLE_DECIMATE_SGIX', 0x8430 )


def glInitResampleSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""ctypes abstraction layer

We keep rewriting functions as the main entry points change,
so let's just localise the changes here...
"""
import ctypes, logging
log = logging.getLogger( 'OpenGL.ctypes' )
log.setLevel( logging.DEBUG )
ctypes_version = [
	int(x) for x in ctypes.__version__.split('.')
]
from ctypes import util


def loadLibrary( dllType, name, mode = ctypes.RTLD_GLOBAL ):
	"""Load a given library by name with the given mode
	
	dllType -- the standard ctypes pointer to a dll type, such as
		ctypes.cdll or ctypes.windll or the underlying ctypes.CDLL or 
		ctypes.WinDLL classes.
	name -- a short module name, e.g. 'GL' or 'GLU'
	mode -- ctypes.RTLD_GLOBAL or ctypes.RTLD_LOCAL,
		controls whether the module resolves names via other
		modules already loaded into this process.  GL modules
		generally need to be loaded with GLOBAL flags
	
	returns the ctypes C-module object
	"""
	if ctypes_version > [0,9,9,3]:
		if isinstance( dllType, ctypes.LibraryLoader ):
			dllType = dllType._dlltype
		fullName = None
		try:
			fullName = util.find_library( name )
			if fullName is not None:
				name = fullName
		except Exception, err:
			log.info( '''Failed on util.find_library( %r ): %s''', name, err )
			# Should the call fail, we just try to load the base filename...
			pass
		try:
			return dllType( name, mode )
		except Exception, err:
			err.args += (name,fullName)
			raise
	elif ctypes_version == [0,9,9,3]:
		return dllType.find( name, mode )
	else:
		raise RuntimeError(
			"""Unsupported ctypes version (%s), please upgrade to at least ctypes 0.9.9.3"""%(
				".".join( [str(x) for x in ctypes_version] ),
			)
		)

def buildFunction( functionType, name, dll ):
	"""Abstract away the ctypes function-creation operation"""
	if ctypes_version > [0,9,9,3]:
		return functionType( (name, dll), )
	elif ctypes_version == [0,9,9,3]:
		return functionType( name, dll )
	else:
		raise RuntimeError(
			"""Unsupported ctypes version (%s), please upgrade to at least ctypes 0.9.9.3"""%(
				".".join( [str(x) for x in ctypes_version] ),
			)
		)




'''OpenGL extension SGIX.list_priority

Overview (from the spec)
	
	This extension provides a mechanism for specifying the relative
	importance of display lists.  This information can be used by
	an OpenGL implementation to guide the placement of display
	list data in a storage hierarchy.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/list_priority.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_list_priority'
GL_LIST_PRIORITY_SGIX = constant.Constant( 'GL_LIST_PRIORITY_SGIX', 0x8182 )
glGetListParameterfvSGIX = platform.createExtensionFunction( 
	'glGetListParameterfvSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glGetListParameterfvSGIX( GLuint(list), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('list', 'pname', 'params',),
)

glGetListParameterivSGIX = platform.createExtensionFunction( 
	'glGetListParameterivSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glGetListParameterivSGIX( GLuint(list), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('list', 'pname', 'params',),
)

glListParameterfSGIX = platform.createExtensionFunction( 
	'glListParameterfSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, constants.GLfloat,),
	doc = 'glListParameterfSGIX( GLuint(list), GLenum(pname), GLfloat(param) ) -> None',
	argNames = ('list', 'pname', 'param',),
)

glListParameterfvSGIX = platform.createExtensionFunction( 
	'glListParameterfvSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLfloatArray,),
	doc = 'glListParameterfvSGIX( GLuint(list), GLenum(pname), GLfloatArray(params) ) -> None',
	argNames = ('list', 'pname', 'params',),
)

glListParameteriSGIX = platform.createExtensionFunction( 
	'glListParameteriSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, constants.GLint,),
	doc = 'glListParameteriSGIX( GLuint(list), GLenum(pname), GLint(param) ) -> None',
	argNames = ('list', 'pname', 'param',),
)

glListParameterivSGIX = platform.createExtensionFunction( 
	'glListParameterivSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLenum, arrays.GLintArray,),
	doc = 'glListParameterivSGIX( GLuint(list), GLenum(pname), GLintArray(params) ) -> None',
	argNames = ('list', 'pname', 'params',),
)


def glInitListPrioritySGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



#! /usr/bin/python

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

import numpy


class Bug(object):
    def __init__(self):
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
        glutCreateWindow("Bug")
        
        glutDisplayFunc(self.on_display)
        glutReshapeFunc(self.on_reshape)

        glClearDepth(1.0)
        glEnable(GL_DEPTH_TEST)
        glClearColor(0.0, 0.0, 0.0, 0.0)
        glShadeModel(GL_FLAT)

        glMatrixMode(GL_MODELVIEW)
        glLightfv(GL_LIGHT0, GL_POSITION,  (40.0, 40, 100.0, 0.0))
        glLightfv(GL_LIGHT0, GL_DIFFUSE, (0.99, 0.99, 0.99, 1.0))
        glEnable(GL_LIGHT0)
        glEnable(GL_LIGHTING)
        glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE)
        glEnable(GL_COLOR_MATERIAL)
        
        self.quad = gluNewQuadric()

    def on_reshape(self, width, height):
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        a = 9.0/min(width, height)
        clipping_planes = (a*width, a*height)
        glFrustum(-clipping_planes[0], clipping_planes[0], -clipping_planes[1], clipping_planes[1], 50.0, 150.0)


    def on_display(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        # BEGIN INTERESTING PART
        # Set up the model view matrix
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        transf = numpy.identity(4, dtype=numpy.float32)
        s = numpy.sin(0.1)
        c = numpy.cos(0.1)
        # A Given's rotation about the z-axis
        transf[0,0] = c
        transf[0,1] = s
        transf[1,0] = -s
        transf[1,1] = c
        # A translation
        transf[0,3] = 2.5
        transf[2,3] = -80
        print glGetFloatv(GL_MODELVIEW_MATRIX)
        print transf.transpose()
        # This doesn't work:
        glMultMatrixf(transf.transpose())
        # This does work:
        #glMultMatrixf(transf.transpose().copy())
        print glGetFloatv(GL_MODELVIEW_MATRIX)
        # END INTERESTING PART

        glColor3f(0.6, 0.8, 0.3)
        gluSphere(self.quad, 10, 8, 8)
        glutSwapBuffers()



if __name__ == '__main__':
    glutInit(sys.argv)
    Bug()
    glutMainLoop ()



'''OpenGL extension ARB.vertex_array_object

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/vertex_array_object.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_vertex_array_object'
GL_VERTEX_ARRAY_BINDING = constant.Constant( 'GL_VERTEX_ARRAY_BINDING', 0x85B5 )
glBindVertexArray = platform.createExtensionFunction( 
	'glBindVertexArray', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glBindVertexArray( GLuint(array) ) -> None',
	argNames = ('array',),
)

glDeleteVertexArrays = platform.createExtensionFunction( 
	'glDeleteVertexArrays', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glDeleteVertexArrays( GLsizei(n), GLuintArray(arrays) ) -> None',
	argNames = ('n', 'arrays',),
)

glGenVertexArrays = platform.createExtensionFunction( 
	'glGenVertexArrays', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLuintArray,),
	doc = 'glGenVertexArrays( GLsizei(n), GLuintArray(arrays) ) -> None',
	argNames = ('n', 'arrays',),
)

glIsVertexArray = platform.createExtensionFunction( 
	'glIsVertexArray', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLuint,),
	doc = 'glIsVertexArray( GLuint(array) ) -> constants.GLboolean',
	argNames = ('array',),
)


def glInitVertexArrayObjectARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Run-time calculation of offset into Python string structure

Does a scan to find the digits of pi in a string structure
in order to produce an offset that can be used to produce 
data-pointers from Python strings.

Porting note:
	
	Currently this uses id( str a ) to get the base address
	of the Python string.  Python implementations where id( a )
	is *not* the memory address of the string will not work!
"""
import ctypes
PI_DIGITS = '31415926535897931'

def calculateOffset( ):
	"""Calculates the data-pointer offset for strings
	
	This does a sequential scan for 100 bytes from the id
	of a string to find special data-value stored in the
	string (the digits of PI).  It produces a dataPointer
	function which adds that offset to the id of the 
	passed strings.
	"""
	finalOffset = None
	a = PI_DIGITS
	# XXX NOT portable across Python implmentations!!!
	initial = id(a)
	targetType = ctypes.POINTER( ctypes.c_char )
	for offset in range( 100 ):
		vector = ctypes.cast( initial+offset,targetType )
		allMatched = True
		for index,digit in enumerate( a ):
			if vector[index] != digit:
				allMatched = False
				break
		if allMatched:
			finalOffset = offset
			break
	if finalOffset is not None:
		def dataPointer( data ):
			"""Return the data-pointer from the array using calculated offset
			
			data -- a Python string
			
			Returns the raw data-pointer to the internal buffer of the passed string
			"""
			if not isinstance( data, str ):
				raise TypeError(
					"""This function can only handle Python strings!  Got %s"""%(
						type(data),
					)
				)
			return id(data) + finalOffset
		# just for later reference...
		dataPointer.offset = finalOffset 
		return dataPointer
	raise RuntimeError(
		"""Unable to determine dataPointer offset for strings!"""
	)

dataPointer = calculateOffset()

if __name__ == "__main__":
	a  = 'this'
	print id(a), dataPointer( a ), dataPointer(a) - id(a)
	



'''OpenGL extension EXT.texture_sRGB

Overview (from the spec)
	
	Conventional texture formats assume a linear color space.  So for
	a conventional internal texture format such as GL_RGB8, the 256
	discrete values for each 8-bit color component map linearly and
	uniformly to the [0,1] range.
	
	The sRGB color space is based on typical (non-linear) monitor
	characteristics expected in a dimly lit office.  It has been
	standardized by the International Electrotechnical Commission (IEC)
	as IEC 61966-2-1. The sRGB color space roughly corresponds to 2.2
	gamma correction.
	
	This extension adds a few new uncompressed and compressed color
	texture formats with sRGB color components.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/texture_sRGB.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_sRGB'
GL_SRGB_EXT = constant.Constant( 'GL_SRGB_EXT', 0x8C40 )
GL_SRGB8_EXT = constant.Constant( 'GL_SRGB8_EXT', 0x8C41 )
GL_SRGB_ALPHA_EXT = constant.Constant( 'GL_SRGB_ALPHA_EXT', 0x8C42 )
GL_SRGB8_ALPHA8_EXT = constant.Constant( 'GL_SRGB8_ALPHA8_EXT', 0x8C43 )
GL_SLUMINANCE_ALPHA_EXT = constant.Constant( 'GL_SLUMINANCE_ALPHA_EXT', 0x8C44 )
GL_SLUMINANCE8_ALPHA8_EXT = constant.Constant( 'GL_SLUMINANCE8_ALPHA8_EXT', 0x8C45 )
GL_SLUMINANCE_EXT = constant.Constant( 'GL_SLUMINANCE_EXT', 0x8C46 )
GL_SLUMINANCE8_EXT = constant.Constant( 'GL_SLUMINANCE8_EXT', 0x8C47 )
GL_COMPRESSED_SRGB_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_EXT', 0x8C48 )
GL_COMPRESSED_SRGB_ALPHA_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_EXT', 0x8C49 )
GL_COMPRESSED_SLUMINANCE_EXT = constant.Constant( 'GL_COMPRESSED_SLUMINANCE_EXT', 0x8C4A )
GL_COMPRESSED_SLUMINANCE_ALPHA_EXT = constant.Constant( 'GL_COMPRESSED_SLUMINANCE_ALPHA_EXT', 0x8C4B )
GL_COMPRESSED_SRGB_S3TC_DXT1_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_S3TC_DXT1_EXT', 0x8C4C )
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT1_EXT', 0x8C4D )
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT3_EXT', 0x8C4E )
GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA_S3TC_DXT5_EXT', 0x8C4F )


def glInitTextureSrgbEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.shadow

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shadow to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.shadow import *
### END AUTOGENERATED SECTION
from OpenGL.GL import glget 
glget.TEX_PARAMETER_SIZES[ GL_TEXTURE_COMPARE_MODE_ARB ] = (1,)
glget.TEX_PARAMETER_SIZES[ GL_TEXTURE_COMPARE_FUNC_ARB ] = (1,)



"""Utility module to parse a Feedback buffer"""
from OpenGL import contextdata
from OpenGL.raw import GL as simple
from OpenGL.GL import glget

def parseFeedback( buffer, entryCount ):
	"""Parse the feedback buffer into Python object records"""
	bufferIndex = 0
	result = []
	getVertex = createGetVertex( )
	while bufferIndex < entryCount:
		token = int(buffer[bufferIndex])
		bufferIndex += 1
		if SINGLE_VERTEX_TOKENS.has_key( token):
			vData, bufferIndex = getVertex( buffer, bufferIndex )
			result.append( (SINGLE_VERTEX_TOKENS.get(token), Vertex(*vData)) )
		elif DOUBLE_VERTEX_TOKENS.has_key( token ):
			vData, bufferIndex = getVertex( buffer, bufferIndex )
			vData2, bufferIndex = getVertex( buffer, bufferIndex )
			result.append( (
				DOUBLE_VERTEX_TOKENS.get(token), 
				Vertex(*vData),
				Vertex(*vData2),
			) )
		elif token == simple.GL_PASS_THROUGH_TOKEN:
			result.append( (simple.GL_PASS_THROUGH_TOKEN, buffer[bufferIndex]))
			bufferIndex += 1
		elif token == simple.GL_POLYGON_TOKEN:
			temp = [simple.GL_POLYGON_TOKEN]
			count = int(buffer[bufferIndex])
			bufferIndex += 1
			for item in range(count):
				vData,bufferIndex = getVertex( buffer, bufferIndex )
				temp.append( Vertex(*vData))
			result.append( tuple(temp))
		else:
			raise ValueError( 
				"""Unrecognised token %r in feedback stream"""%(token,)
			)
	return result

SINGLE_VERTEX_TOKENS = {
	simple.GL_BITMAP_TOKEN: simple.GL_BITMAP_TOKEN,
	simple.GL_COPY_PIXEL_TOKEN: simple.GL_COPY_PIXEL_TOKEN,
	simple.GL_DRAW_PIXEL_TOKEN: simple.GL_DRAW_PIXEL_TOKEN,
	simple.GL_POINT_TOKEN: simple.GL_POINT_TOKEN,
}
DOUBLE_VERTEX_TOKENS = {
	simple.GL_LINE_TOKEN: simple.GL_LINE_TOKEN,
	simple.GL_LINE_RESET_TOKEN: simple.GL_LINE_RESET_TOKEN,
}
class Vertex( object ):
	"""Simplistic holder for vertex data from a feedback buffer"""
	__slots__ = ('vertex','color','texture')
	def __init__( self, vertex,color=None,texture=None):
		"""Store values for access"""
		self.vertex = vertex 
		self.color = color 
		self.texture = texture 
def createGetVertex( ):
	mode = contextdata.getValue( "GL_FEEDBACK_BUFFER_TYPE" )
	indexMode = glget.glGetBoolean( simple.GL_INDEX_MODE )
	colorSize = [ 4,1 ][ int(indexMode) ]
	if mode in (simple.GL_2D,simple.GL_3D):
		if mode == simple.GL_2D:
			size = 2
		else:
			size = 3
		def getVertex( buffer, bufferIndex ):
			end = bufferIndex+size
			return (buffer[bufferIndex:end],None,None),end 
	elif mode == simple.GL_3D_COLOR:
		def getVertex( buffer, bufferIndex ):
			end = bufferIndex+3
			colorEnd = end + colorSize
			return (buffer[bufferIndex:end],buffer[end:colorEnd],None),colorEnd 
	else:
		if mode == simple.GL_3D_COLOR_TEXTURE:
			size = 3
		else:
			size = 4
		def getVertex( buffer, bufferIndex ):
			end = bufferIndex+size
			colorEnd = end + colorSize
			textureEnd = colorEnd + 4
			return (buffer[bufferIndex:end],buffer[end:colorEnd],buffer[colorEnd:textureEnd]),textureEnd
	return getVertex



'''OpenGL extension SGIX.igloo_interface

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/igloo_interface.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_igloo_interface'

glIglooInterfaceSGIX = platform.createExtensionFunction( 
	'glIglooInterfaceSGIX', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, ctypes.c_void_p,),
	doc = 'glIglooInterfaceSGIX( GLenum(pname), c_void_p(params) ) -> None',
	argNames = ('pname', 'params',),
)


def glInitIglooInterfaceSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ATI.map_object_buffer

Overview (from the spec)
	
	This extension provides a mechanism for an application to obtain
	the virtual address of an object buffer. This allows the
	application to directly update the contents of an object buffer
	and avoid any intermediate copies.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/map_object_buffer.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_map_object_buffer'

glMapObjectBufferATI = platform.createExtensionFunction( 
	'glMapObjectBufferATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=ctypes.c_void_p, 
	argTypes=(constants.GLuint,),
	doc = 'glMapObjectBufferATI( GLuint(buffer) ) -> ctypes.c_void_p',
	argNames = ('buffer',),
)

glUnmapObjectBufferATI = platform.createExtensionFunction( 
	'glUnmapObjectBufferATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint,),
	doc = 'glUnmapObjectBufferATI( GLuint(buffer) ) -> None',
	argNames = ('buffer',),
)


def glInitMapObjectBufferATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.texture_expand_normal

Overview (from the spec)
	
	This extension provides a remapping mode where unsigned texture
	components (in the range [0,1]) can be treated as though they
	contained signed data (in the range [-1,+1]).  This allows
	applications to easily encode signed data into unsigned texture
	formats.
	
	The functionality of this extension is nearly identical to the
	EXPAND_NORMAL_NV remapping mode provided in the NV_register_combiners
	extension, although it applies even if register combiners are used.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/texture_expand_normal.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_texture_expand_normal'
GL_TEXTURE_UNSIGNED_REMAP_MODE_NV = constant.Constant( 'GL_TEXTURE_UNSIGNED_REMAP_MODE_NV', 0x888F )


def glInitTextureExpandNormalNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.present_video

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/present_video.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_present_video'
GL_FRAME_NV = constant.Constant( 'GL_FRAME_NV', 0x8E26 )
GL_FIELDS_NV = constant.Constant( 'GL_FIELDS_NV', 0x8E27 )
GL_CURRENT_TIME_NV = constant.Constant( 'GL_CURRENT_TIME_NV', 0x8E28 )
GL_NUM_FILL_STREAMS_NV = constant.Constant( 'GL_NUM_FILL_STREAMS_NV', 0x8E29 )
GL_PRESENT_TIME_NV = constant.Constant( 'GL_PRESENT_TIME_NV', 0x8E2A )
GL_PRESENT_DURATION_NV = constant.Constant( 'GL_PRESENT_DURATION_NV', 0x8E2B )


def glInitPresentVideoNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""ctypes sized data-arrays as a data-formatmechanism

XXX we have to use _ctypes.Array as the type descriminator,
would be nice to have it available from the public module
"""
REGISTRY_NAME = 'ctypesarrays'
import ctypes, _ctypes

from OpenGL import constants, constant
from OpenGL.arrays import formathandler
import operator

class CtypesArrayHandler( formathandler.FormatHandler ):
	"""Ctypes Array-type-specific data-type handler for OpenGL"""
	from_param = staticmethod( ctypes.byref )
	dataPointer = staticmethod( ctypes.addressof )
	HANDLED_TYPES = (_ctypes.Array, )
	def voidDataPointer( cls, value ):
		"""Given value in a known data-pointer type, return void_p for pointer"""
		return ctypes.byref( value )
	def zeros( self, dims, typeCode ):
		"""Return Numpy array of zeros in given size"""
		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
		for dim in dims:
			type *= dim 
		return type() # should expicitly set to 0s
	def ones( self, dims, typeCode='d' ):
		"""Return numpy array of ones in given size"""
		raise NotImplementedError( """Haven't got a good ones implementation yet""" )
##		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
##		for dim in dims:
##			type *= dim 
##		return type() # should expicitly set to 0s
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
		if result is not None:
			return result
		raise TypeError(
			"""Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
				value._type_, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
			)
		)
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		dims = 1
		for base in self.types( value ):
			length = getattr( base, '_length_', None)
			if length is not None:
				dims *= length
		return dims 
	def arrayByteCount( self, value, typeCode = None ):
		"""Given a data-value, calculate number of bytes required to represent"""
		return ctypes.sizeof( value )
	def types( self, value ):
		"""Produce iterable producing all composite types"""
		dimObject = value
		while dimObject is not None:
			yield dimObject
			dimObject = getattr( dimObject, '_type_', None )
			if isinstance( dimObject, (str,unicode)):
				dimObject = None 
	def dims( self, value ):
		"""Produce iterable of all dimensions"""
		for base in self.types( value ):
			length = getattr( base, '_length_', None)
			if length is not None:
				yield length
	def asArray( self, value, typeCode=None ):
		"""Convert given value to an array value of given typeCode"""
		return value
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		for dim in self.dims( value ):
			pass 
		return dim
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		return tuple( self.dims(value) )


ARRAY_TO_GL_TYPE_MAPPING = {
	constants.GLdouble: constants.GL_DOUBLE,
	constants.GLfloat: constants.GL_FLOAT,
	constants.GLint: constants.GL_INT,
	constants.GLuint: constants.GL_UNSIGNED_INT,
	constants.GLshort: constants.GL_SHORT,
	constants.GLushort: constants.GL_UNSIGNED_SHORT,
		
	constants.GLchar: constants.GL_CHAR,
	constants.GLbyte: constants.GL_BYTE,
	constants.GLubyte: constants.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
	constants.GL_DOUBLE: constants.GLdouble,
	constants.GL_FLOAT: constants.GLfloat,
	constants.GL_INT: constants.GLint,
	constants.GL_UNSIGNED_INT: constants.GLuint,
	constants.GL_SHORT: constants.GLshort,
	constants.GL_UNSIGNED_SHORT: constants.GLushort,
		
	constants.GL_CHAR: constants.GLchar,
	constants.GL_BYTE: constants.GLbyte,
	constants.GL_UNSIGNED_BYTE: constants.GLubyte,
}



'''OpenGL extension MESA.window_pos

Overview (from the spec)
	
	In order to set the current raster position to a specific window
	coordinate with the RasterPos command, the modelview matrix, projection
	matrix and viewport must be set very carefully.  Furthermore, if the
	desired window coordinate is outside of the window's bounds one must
	rely on a subtle side-effect of the Bitmap command in order to circumvent
	frustum clipping.
	
	This extension provides a set of functions to directly set the
	current raster position, bypassing the modelview matrix, the
	projection matrix and the viewport to window mapping.  Furthermore,
	clip testing is not performed.
	
	This greatly simplifies the process of setting the current raster
	position to a specific window coordinate prior to calling DrawPixels,
	CopyPixels or Bitmap.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/MESA/window_pos.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_MESA_window_pos'

glWindowPos2dMESA = platform.createExtensionFunction( 
	'glWindowPos2dMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble,),
	doc = 'glWindowPos2dMESA( GLdouble(x), GLdouble(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2dvMESA = platform.createExtensionFunction( 
	'glWindowPos2dvMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glWindowPos2dvMESA( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos2fMESA = platform.createExtensionFunction( 
	'glWindowPos2fMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat,),
	doc = 'glWindowPos2fMESA( GLfloat(x), GLfloat(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2fvMESA = platform.createExtensionFunction( 
	'glWindowPos2fvMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glWindowPos2fvMESA( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos2iMESA = platform.createExtensionFunction( 
	'glWindowPos2iMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint,),
	doc = 'glWindowPos2iMESA( GLint(x), GLint(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2ivMESA = platform.createExtensionFunction( 
	'glWindowPos2ivMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glWindowPos2ivMESA( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos2sMESA = platform.createExtensionFunction( 
	'glWindowPos2sMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort,),
	doc = 'glWindowPos2sMESA( GLshort(x), GLshort(y) ) -> None',
	argNames = ('x', 'y',),
)

glWindowPos2svMESA = platform.createExtensionFunction( 
	'glWindowPos2svMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glWindowPos2svMESA( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3dMESA = platform.createExtensionFunction( 
	'glWindowPos3dMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glWindowPos3dMESA( GLdouble(x), GLdouble(y), GLdouble(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3dvMESA = platform.createExtensionFunction( 
	'glWindowPos3dvMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glWindowPos3dvMESA( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3fMESA = platform.createExtensionFunction( 
	'glWindowPos3fMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glWindowPos3fMESA( GLfloat(x), GLfloat(y), GLfloat(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3fvMESA = platform.createExtensionFunction( 
	'glWindowPos3fvMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glWindowPos3fvMESA( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3iMESA = platform.createExtensionFunction( 
	'glWindowPos3iMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glWindowPos3iMESA( GLint(x), GLint(y), GLint(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3ivMESA = platform.createExtensionFunction( 
	'glWindowPos3ivMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glWindowPos3ivMESA( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos3sMESA = platform.createExtensionFunction( 
	'glWindowPos3sMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glWindowPos3sMESA( GLshort(x), GLshort(y), GLshort(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glWindowPos3svMESA = platform.createExtensionFunction( 
	'glWindowPos3svMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glWindowPos3svMESA( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos4dMESA = platform.createExtensionFunction( 
	'glWindowPos4dMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glWindowPos4dMESA( GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w) ) -> None',
	argNames = ('x', 'y', 'z', 'w',),
)

glWindowPos4dvMESA = platform.createExtensionFunction( 
	'glWindowPos4dvMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glWindowPos4dvMESA( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos4fMESA = platform.createExtensionFunction( 
	'glWindowPos4fMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glWindowPos4fMESA( GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w) ) -> None',
	argNames = ('x', 'y', 'z', 'w',),
)

glWindowPos4fvMESA = platform.createExtensionFunction( 
	'glWindowPos4fvMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glWindowPos4fvMESA( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos4iMESA = platform.createExtensionFunction( 
	'glWindowPos4iMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glWindowPos4iMESA( GLint(x), GLint(y), GLint(z), GLint(w) ) -> None',
	argNames = ('x', 'y', 'z', 'w',),
)

glWindowPos4ivMESA = platform.createExtensionFunction( 
	'glWindowPos4ivMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glWindowPos4ivMESA( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glWindowPos4sMESA = platform.createExtensionFunction( 
	'glWindowPos4sMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glWindowPos4sMESA( GLshort(x), GLshort(y), GLshort(z), GLshort(w) ) -> None',
	argNames = ('x', 'y', 'z', 'w',),
)

glWindowPos4svMESA = platform.createExtensionFunction( 
	'glWindowPos4svMESA', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glWindowPos4svMESA( GLshortArray(v) ) -> None',
	argNames = ('v',),
)


def glInitWindowPosMESA():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension IBM.cull_vertex

Overview (from the spec)
	
	IBM_cull_vertex provides a subset of the vertex culling functionality
	found in EXT_cull_vertex without providing a guarantee that faces will
	be culled because of it.  EXT_cull_vertex is a technically superior
	solution, but the vertex culling aspect of IBM_cull_vertex provides
	generally useful function cheaply (without imposing the mandated
	culling found in EXT_cull_vertex).

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/IBM/cull_vertex.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_IBM_cull_vertex'



def glInitCullVertexIBM():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Simple plug-in mechanism to provide replacement for setuptools plugins"""

class Plugin( object ):
	"""Base class for plugins to be loaded"""
	loaded = False
	def __init__( self, name, import_path, check = None ):
		"""Register the plug-in"""
		self.name = name 
		self.import_path = import_path
		self.check = check
		self.registry.append( self )
	def load( self ):
		"""Attempt to load and return our entry point"""
		return importByName( self.import_path )
	@classmethod
	def match( cls, *args ):
		"""Match to return the plugin which is appropriate to load"""
	@classmethod
	def all( cls ):
		"""Iterate over all registered plugins"""
		return cls.registry[:]
	
def importByName( fullName ):
	"""Import a class by name"""
	name = fullName.split(".")
	moduleName = name[:-1]
	className = name[-1]
	module = __import__( ".".join(moduleName), {}, {}, moduleName)
	return getattr( module, className )

		
class PlatformPlugin( Plugin ):
	"""Platform-level plugin registration"""
	registry = []
	@classmethod
	def match( cls, key ):
		"""Determine what platform module to load
		
		key -- (sys.platform,os.name) key to load 
		"""
		for plugin in cls.registry:
			if plugin.name in key:
				return plugin
		raise KeyError( """No platform plugin registered for %s"""%(key,))

class FormatHandler( Plugin ):
	"""Data-type storage-format handler"""
	registry = []
	@classmethod
	def match( cls, value ):
		"""Lookup appropriate handler based on value (a type)"""
		key = '%s.%s'%( value.__module__, value.__name__ )
		for plugin in cls.registry:
			set = getattr( plugin, 'check', ())
			if set and key in set:
				return plugin
		return None
